//------------------------------------------------------------------------------
//
//  npengine.c
//
//  ANTz application based on NPE
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010-2011. All rights reserved.
//
//------------------------------------------------------------------------------

#include "npengine.h"

#include <math.h>

#include "../io/npgl.h"
#include "../npdata.h"


void UpdateNode (NPnodePtr node, void* dataRef);

void UpdateNode (NPnodePtr node);
void UpdateNodeDefault (NPnodePtr node);
void UpdateNodeCamera (NPnodePtr node);
void UpdateNodeSurface (NPnodePtr node);
void UpdateNodePoints (NPnodePtr node);
void UpdateNodePin (NPnodePtr node);
void UpdateNodeVideo (NPnodePtr node);
void UpdateNodeGrid (NPnodePtr node);

void UpdateNodeData (NPnodePtr node);


//------------------------------------------------------------------------------
void npInitEngine (void* dataRef)
{

}


//------------------------------------------------------------------------------
void npCloseEngine (void* dataRef)
{

}

//------------------------------------------------------------------------------
void UpdateNodeChild (NPnodePtr node)
{
	int i = 0;

	for (i=0; i < node->childCount; i++)
	{
		UpdateNodeChild (node->child[i]);			//recursively calls this function

		UpdateNodeData (node->child[i]);
		UpdateNode(node->child[i]);
	}
}


//------------------------------------------------------------------------------
void UpdateNodeData (NPnodePtr node)
{
	int colorShift = 0;

	GLfloat scale = 0;

	NPubyteRGBA colorTmp;

	pData data = (pData) npDataGetDataRef();

	colorTmp.r = 127;
	colorTmp.g = 127;
	colorTmp.b = 127;
	colorTmp.a = 127;

	if (node->freeze || node->type != kNodePin)	//do not update if frozen
		return;

	if (node->branchLevel == 0)
	{
		if (node->channel.x)
			node->translate.x = data->io.channel[node->channel.x][data->io.channelIndex];

		if (node->channel.y)
			node->translate.y = data->io.channel[node->channel.y][data->io.channelIndex];

		if (node->channel.z)
		{
			// retrieve original color from index
			SetIndexColor (&node->color, &node->colorIndex);

			colorShift = (int)data->io.channel[node->channel.z][data->io.channelIndex];
			if (colorShift < 0)
				colorShift = 0;
			if (colorShift > 255)
				colorShift = 255;

			colorTmp.r = (GLubyte) ((GLfloat)node->color.g * ((GLfloat)colorShift / 255.0f) + (GLfloat)node->color.r * ( 1.0f - ((GLfloat)colorShift / 255.0f)) );
			colorTmp.g = (GLubyte) ((GLfloat)node->color.b * ((GLfloat)colorShift / 255.0f) + (GLfloat)node->color.g * ( 1.0f - ((GLfloat)colorShift / 255.0f)) );
			colorTmp.b = (GLubyte) ((GLfloat)node->color.r * ((GLfloat)colorShift / 255.0f) + (GLfloat)node->color.b * ( 1.0f - ((GLfloat)colorShift / 255.0f)) );

			node->color.r = colorTmp.r;
			node->color.g = colorTmp.g;
			node->color.b = colorTmp.b;
		}
	}
	else
	{
		if (node->channel.x)
			node->rotateRad.x = data->io.channel[node->channel.x][data->io.channelIndex] 
								* kPos2PI / 256.0f;

		if (node->channel.y)
		{
			scale = 0.35f + ((GLfloat)data->io.channel[node->channel.y][data->io.channelIndex]) / 255.0;

			node->scale.x = scale;
			node->scale.y = scale;
			node->scale.z = scale;
		}


		if (node->channel.z)
		{
			// retrieve original color from index
			SetIndexColor (&node->color, &node->colorIndex);

			colorShift = (int)data->io.channel[node->channel.z][data->io.channelIndex];
			if (colorShift < 0)
				colorShift = 0;
			if (colorShift > 255)
				colorShift = 255;

			colorTmp.r = (GLubyte) ((GLfloat)node->color.g * ((GLfloat)colorShift / 255.0f) + (GLfloat)node->color.r * ( 1.0f - ((GLfloat)colorShift / 255.0f)) );
			colorTmp.g = (GLubyte) ((GLfloat)node->color.b * ((GLfloat)colorShift / 255.0f) + (GLfloat)node->color.g * ( 1.0f - ((GLfloat)colorShift / 255.0f)) );
			colorTmp.b = (GLubyte) ((GLfloat)node->color.r * ((GLfloat)colorShift / 255.0f) + (GLfloat)node->color.b * ( 1.0f - ((GLfloat)colorShift / 255.0f)) );

			node->color.r = colorTmp.r;
			node->color.g = colorTmp.g;
			node->color.b = colorTmp.b;
		}
	}
}

// iterate through nodes update translation rotation and scale
//------------------------------------------------------------------------------
void npEngineUpdate (void* dataRef)
{
	int i = 0;

	pData data = (pData) dataRef;

	NPnodePtr node = NULL;

	for (i=0; i < data->map.nodeCount; i++)
	{
		node = data->map.node[i];

		UpdateNodeData (node);		// move this to data.c						debug, zz

		if (node->childCount)
			UpdateNodeChild(node);

		UpdateNode (node);
	}
}


//------------------------------------------------------------------------------
void UpdateNode (NPnodePtr node)
{
	if (node->freeze)			//do not update if frozen
		return;

	switch (node->type)
	{
		case kNodeDefault : UpdateNodeDefault (node); break;
		case kNodeCamera : UpdateNodeCamera (node); break;
		case kNodeSurface : UpdateNodeSurface (node); break;
		case kNodePoints : UpdateNodePoints (node); break;
		case kNodePin : UpdateNodePin (node); break;
		case kNodeVideo : UpdateNodeVideo (node); break;
		case kNodeGrid : UpdateNodeGrid (node); break;
	}
}


//------------------------------------------------------------------------------
void UpdateNodeDefault (NPnodePtr node)
{
	GLfloat sinX, cosX, sinZ, cosZ;

	pData data = (pData) npDataGetDataRef();

	// update rotation about XYZ axis and keep inbounds of 2 PI
	node->rotateRad.x += node->rotateRate.x * 2.0f;	// * 2 to make faster then cam
	if (node->rotateRad.x >= kPos2PI)
		node->rotateRad.x -= kPos2PI;
	if (node->rotateRad.x <= kNeg2PI)
		node->rotateRad.x += kPos2PI;

	node->rotateRad.y += node->rotateRate.y * 2.0f;
	if (node->rotateRad.y >= kPos2PI)
		node->rotateRad.y -= kPos2PI;
	if (node->rotateRad.y <= kNeg2PI)
		node->rotateRad.y += kPos2PI;

	node->rotateRad.z += node->rotateRate.z * 2.0f;
	if (node->rotateRad.z >= kPos2PI)
		node->rotateRad.z -= kPos2PI;
	if (node->rotateRad.z <= kNeg2PI)
		node->rotateRad.z += kPos2PI;

	//calculate the rotation vector		// optimize with lookup tables, zz
	sinX = (GLfloat)sin(node->rotateRad.x);
	cosX = (GLfloat)cos(node->rotateRad.x);
	sinZ = (GLfloat)sin(node->rotateRad.z);
	cosZ = (GLfloat)cos(node->rotateRad.z);

//	node->rotate.x = sinZ * cosX;
//	node->rotate.y = cosZ * cosX;
//	node->rotate.z = -sinX;

	node->rotate.angle = node->rotateRad.z * 57.29578f;
	node->rotate.x = 0.0f;
	node->rotate.y = 0.0f;
	node->rotate.z = 1.0f;

	//update XYZ scale
	node->scale.x = node->scale.x * (1.0f + node->scaleRate.x);
	node->scale.y = node->scale.y * (1.0f + node->scaleRate.y);
	node->scale.z = node->scale.z * (1.0f + node->scaleRate.z);
	
	//update the translateVect for selected objects
/*	node->translateVect.x = sinZ * node->translateRate.y
						  + cosZ * node->translateRate.x;

	node->translateVect.y = cosZ * node->translateRate.y
						  - sinZ * node->translateRate.x;

	//z ignores the angles just move in global coordinates, 0.5 to reduce speed
	node->translateVect.z = node->translateRate.z * 0.5f;

	//update the position by adding the vector to existing position
	node->translate.x += node->translateVect.x;
	node->translate.y += node->translateVect.y;
	node->translate.z += node->translateVect.z;
*/

	// update mouse movement
	if (node->selected || node == data->map.activeNode)
	{
		node->translate.x += data->io.mouseDeltaX * 0.03;
		node->translate.y += data->io.mouseDeltaY * -0.03;		// invert the Y axes
		node->translate.z += node->translateRate.z;
	}

	node->translate.x += node->translateRate.x;
	node->translate.y += node->translateRate.y;
	node->translate.z += node->translateRate.z;

	//limit from going below the surface
	if (node->triggerOnLow.x)
		if (node->translate.x < node->triggerLevelLow.x)											// debug, zz
			node->translate.x = node->triggerLevelLow.x;
	if (node->triggerOnHi.x)
		if (node->translate.x > node->triggerLevelHi.x)											// debug, zz
			node->translate.x = node->triggerLevelHi.x;
	if (node->triggerOnLow.y)
		if (node->translate.y < node->triggerLevelLow.y)											// debug, zz
			node->translate.y = node->triggerLevelLow.y;
	if (node->triggerOnHi.y)
		if (node->translate.y > node->triggerLevelHi.y)											// debug, zz
			node->translate.y = node->triggerLevelHi.y;
	if (node->triggerOnLow.z)
		if (node->translate.z < node->triggerLevelLow.z)											// debug, zz
			node->translate.z = node->triggerLevelLow.z;
	if (node->triggerOnHi.z)
		if (node->translate.z > node->triggerLevelHi.z)											// debug, zz
			node->translate.z = node->triggerLevelHi.z;
}


//------------------------------------------------------------------------------
void UpdateNodeCamera (NPnodePtr node)
{
	GLfloat sinX, cosX, sinZ, cosZ;

	// update rotation about XYZ axis and keep inbounds of 2 PI
	node->rotateRad.x += node->rotateRate.x;
	if (node->rotateRad.x >= kPos2PI / 4.0f)	// limit to +/- 90 deg
		node->rotateRad.x = kPos2PI / 4.0f;
	if (node->rotateRad.x <= kNeg2PI / 4.0f)
		node->rotateRad.x = -kPos2PI / 4.0f;

	node->rotateRad.y += node->rotateRate.y;
	if (node->rotateRad.y >= kPos2PI)
		node->rotateRad.y -= kPos2PI;
	if (node->rotateRad.y <= kNeg2PI)
		node->rotateRad.y += kPos2PI;

	node->rotateRad.z += node->rotateRate.z;
	if (node->rotateRad.z >= kPos2PI)
		node->rotateRad.z -= kPos2PI;
	if (node->rotateRad.z <= kNeg2PI)
		node->rotateRad.z += kPos2PI;

	//calculate the rotation vector		// optimize with lookup tables, zz
	sinX = (GLfloat)sin(node->rotateRad.x);
	cosX = (GLfloat)cos(node->rotateRad.x);
	sinZ = (GLfloat)sin(node->rotateRad.z);
	cosZ = (GLfloat)cos(node->rotateRad.z);

	node->rotate.x = sinZ * cosX;
	node->rotate.y = cosZ * cosX;
	node->rotate.z = -sinX;

	//update XYZ scale
	node->scale.x += node->scaleRate.x;
	node->scale.y += node->scaleRate.y;
	node->scale.z += node->scaleRate.z;
	
	//update the translateVect for selected objects
	node->translateVect.x = sinZ * node->translateRate.y
						  + cosZ * node->translateRate.x;

	node->translateVect.y = cosZ * node->translateRate.y
						  - sinZ * node->translateRate.x;

	//z ignores the angles just move in global coordinates, 0.5 to reduce speed
	node->translateVect.z = node->translateRate.z * 0.5f;

	//update the position by adding the vector to existing position
	node->translate.x += node->translateVect.x;
	node->translate.y += node->translateVect.y;
	node->translate.z += node->translateVect.z;

	//limit from going below the surface
	if (node->triggerOnLow.z)
		if (node->translate.z < node->triggerLevelLow.z)											// debug, zz
			node->translate.z = node->triggerLevelLow.z + 0.0001f;

	//limit from going below the surface
	if (node->triggerOnHi.z)
		if (node->translate.z > node->triggerLevelHi.z)											// debug, zz
			node->translate.z = node->triggerLevelHi.z - 0.0001f;
}


//------------------------------------------------------------------------------
void UpdateNodeSurface (NPnodePtr node)
{
	UpdateNodeDefault (node);
}


//------------------------------------------------------------------------------
void UpdateNodePoints (NPnodePtr node)
{
	UpdateNodeDefault (node);
}


//------------------------------------------------------------------------------
void UpdateNodePin (NPnodePtr node)
{
	pData data = (pData) npDataGetDataRef();

	if (node->branchLevel == 0)
	{
		UpdateNodeDefault (node);
		return;
	}
	
	// update mouse movement if selected
	if (node->selected || node == data->map.activeNode)
	{
			node->rotateRad.x += data->io.mouseDeltaX * 0.01;

			node->scale.x += data->io.mouseDeltaY * -0.01;		// invert the Y axes	
			node->scale.y += data->io.mouseDeltaY * -0.01;		// insert check for active axes, debug, zz
			node->scale.z += data->io.mouseDeltaY * -0.01;
	}

	node->rotateRad.x += node->rotateRate.x * 2.0f;	// * 2 to make faster then cam
	if (node->rotateRad.x >= kPos2PI)
		node->rotateRad.x -= kPos2PI;
	if (node->rotateRad.x <= kNeg2PI)
		node->rotateRad.x += kPos2PI;

	node->rotate.angle = node->rotateRad.x * 57.29578f;

	//update XYZ scale
	node->scale.x += node->scaleRate.x;
	node->scale.y += node->scaleRate.y;
	node->scale.z += node->scaleRate.z;
}


//------------------------------------------------------------------------------
void UpdateNodeVideo (NPnodePtr node)
{
	UpdateNodeDefault (node);
}


//------------------------------------------------------------------------------
void UpdateNodeGrid (NPnodePtr node)
{
	//update XYZ scale
	node->scale.x = node->scale.x * (1.0f + node->scaleRate.x);
	node->scale.y = node->scale.y * (1.0f + node->scaleRate.y);
	node->scale.z = node->scale.z * (1.0f + node->scaleRate.z);
}

