//------------------------------------------------------------------------------
//
//  npdata.cpp
//  npe
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------


#include "npdata.h"
#include "npctrl.h"
#include "data/npnode.h"
#include "io/npkey.h"

#include <stdlib.h>

void* gData;

void npInitDataMap (void* dataRef);
void npInitDataIO (void* dataRef);
void npInitDataCtrl (void* dataRef);


//------------------------------------------------------------------------------
void npInitDataMap(void* dataRef)
{
	int i = 0;

	pData data = (pData) dataRef;

	NPnodePtr node			= NULL;
	NPcameraPtr cameraData	= NULL;
	NPpointsPtr pointsData	= NULL;
	NPpinPtr pinData		= NULL;
	NPvideoPtr videoData	= NULL;
	NPgridPtr gridData		= NULL;

	npInitMap (dataRef);

	data->map.nodeCount		= 0;
	data->map.nodeIndex		= 0;
	npInitNode (dataRef);

	data->map.activeNode	= NULL;
	data->map.selectedNode	= NULL;
	data->map.selectedIndex = 0;

	data->map.selectionSet.x = false;
	data->map.selectionSet.y = false;
	data->map.selectionSet.z = false;
	data->map.selectAll		= false;

	npNodeNew (kNodeDefault, dataRef);			//global parent at index = 0

	// setup our primary camera
	node = npNodeNew (kNodeCamera, dataRef);	//camera at index = 1
	npDataCameraReset (node, data);

	// setup objects
	node = npNodeNew (kNodeGrid, dataRef);

	node->scale.z = 1.5f;

//	npDataPreset (1, dataRef);
	npCtrlCommand (kGraphCamera, dataRef);

	// clear the map
	for (i=0; i < kNPdataTypeMax; i++)
		data->map.typeMap[i] = NULL;
}


//------------------------------------------------------------------------------
void npInitDataIO(void* dataRef)
{
	int i = 0, j = 0;

	pData data = (pData) dataRef;

	npInitKey (dataRef);

	data->io.mouseDeltaX = 0.0f;
	data->io.mouseDeltaY = 0.0f;

	data->io.drawMenu = true;

	data->io.clearFrameBuffer = true;
	data->io.clearIndex = 0;

	data->io.clear.r = 0.0f;
	data->io.clear.g = 0.0f;
	data->io.clear.b = 0.0f;
	data->io.clear.a = 0.0f;

	data->io.axes.x = true;
	data->io.axes.y = true;
	data->io.axes.z = true;

	data->io.camTranslate[0].x = 0.0f;
	data->io.camTranslate[0].y = 0.0f;
	data->io.camTranslate[0].z = 0.0f;
	data->io.camRotate[0].x = 0.0f;
	data->io.camRotate[0].y = 0.0f;
	data->io.camRotate[0].z = 0.0f;

	data->io.camTranslate[1].x = 0.0f;
	data->io.camTranslate[1].y = 0.0f;
	data->io.camTranslate[1].z = 100.0f;
	data->io.camRotate[1].x = kPos2PI / 4.0f;
	data->io.camRotate[1].y = 0.0f;
	data->io.camRotate[1].z = 0.0f;

	data->io.camTranslate[2].x = 0.0f;
	data->io.camTranslate[2].y = -85.0f;
	data->io.camTranslate[2].z = 7.0f;
	data->io.camRotate[2].x = 0.0f;
	data->io.camRotate[2].y = 0.0f;
	data->io.camRotate[2].z = 0.0f;

	data->io.camTranslate[3].x = 85.0f;
	data->io.camTranslate[3].y = 0.0f;
	data->io.camTranslate[3].z = 7.0f;
	data->io.camRotate[3].x = 0.0f;
	data->io.camRotate[3].y = 0.0f;
	data->io.camRotate[3].z = -kPos2PI / 4.0f;


	data->io.camIndex = 0;

	for ( i=0; i < kNPchannelMax; i++)
		for ( j=0; j < kNPchannelSampleMax; j++)
			data->io.channel[i][j] = 0.0f;
	
	data->io.channelCount = 40;
	data->io.channelIndex = 0;
	data->io.channelOldest = 0;
	data->io.channelSize = 0;


	data->io.size = 0;

	// clear the map
	for (i=0; i < kNPdataTypeMax; i++)
		data->io.write[i] = NULL;
}


//------------------------------------------------------------------------------
void npInitDataCtrl(void* dataRef)
{
	
}


//------------------------------------------------------------------------------
void npInitData (int argc, char* argv[], void* dataRef)
{
	pData data = (pData) malloc (sizeof(Data));
	if (data == NULL)
	{
		printf ("\n 4213 error malloc failed cannot write file \n");
		return;
	}
	
	npInitDataMap (data);
	npInitDataIO (data);
	npInitDataCtrl (data);

	dataRef = gData = data;

	// create a fixed array of global data pointers that is thread safe,		ZZ
}


//------------------------------------------------------------------------------
void npCloseData()
{
	pData data = (pData) gData;
	
	free (data);
}


//------------------------------------------------------------------------------
void* npDataGetDataRef()
{
	return gData;
}

//------------------------------------------------------------------------------
int npNewNodeID()
{
	static int nodeID = 0;

	nodeID++;

	return nodeID;
}

//------------------------------------------------------------------------------
void npDataUpdate (void* dataRef)
{
	pData data = (pData) dataRef;

	//update the currentIndex
	data->io.channelIndex++;
	if (data->io.channelIndex >= data->io.channelSize)
		data->io.channelIndex = 0;

//	printf ("\n %d %d", data->io.channelIndex, (int)data->io.channel[10][data->io.channelIndex]);
}

//------------------------------------------------------------------------------
char* npGetDataTypeName (int type)
{
	if (type > 0 )//&& type < kTypeCount)
		return NULL;//(data->map.typeList[type].name);
	else
		return NULL;
}
//------------------------------------------------------------------------------
char* npGetDataTypeDesc (int type)
{
	if (type > 0 )//&& type < kTypeCount)
		return NULL;//(data->map.typeList[type].description);
	else
		return NULL;
}


// loads preset scene using currently selected object as the origin
// if alt is pressed (ie ALT+5) it loads preset at the global origin
//------------------------------------------------------------------------------
void npDataPreset (int preset, void* dataRef)
{
	int i = 0;
	int j = 0;

	NPfloatXYZ	translateOffset;
	NPfloatXYZ	rotateOffset;
	NPfloatXYZ	scaleOffset;

	pData data = (pData) dataRef;
	NPnodePtr node = data->map.node[data->map.nodeIndex];

	NPnodePtr nodeChild = NULL;

	NPpinPtr pinData = NULL;

	// don't offset if alt key held or camera selected
	if (data->io.key.modAlt || data->map.nodeIndex == 1)
	{
		translateOffset.x = 0.0f;
		translateOffset.y = 0.0f;
		translateOffset.z = 0.0f;

		rotateOffset.x = 0.0f;
		rotateOffset.y = 0.0f;
		rotateOffset.z = 0.0f;

		scaleOffset.x = 0.0f;
		scaleOffset.y = 0.0f;
		scaleOffset.z = 0.0f;
	}
	else
	{
					// add code to calc translate, rotate position based on
					// global coordinates think of GL order of operation,	debug zz
		translateOffset.x = node->translate.x;
		translateOffset.y = node->translate.y;
		translateOffset.z = node->translate.z;

		rotateOffset.x = node->rotateRad.x;
		rotateOffset.y = node->rotateRad.y;
		rotateOffset.z = node->rotateRad.z;

		scaleOffset.x = node->scale.x;
		scaleOffset.y = node->scale.y;
		scaleOffset.z = node->scale.z;
	}

	switch (preset)
	{
		case kGraphPresetOne :
					node = npNodeNew (kNodePin, dataRef);
					node->colorIndex = 15;
					nodeChild = node->child[0];
						nodeChild->colorIndex = 14;

					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 14;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 2;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 3;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 4;

					npCtrlCommand (kGraphSelectAll, data);
					npCtrlCommand (kGraphColorUp, data);
					npCtrlCommand (kGraphColorDown, data);
					npCtrlCommand (kGraphSelectAll, data);

					data->map.nodeIndex = data->map.nodeCount - 1;
					data->map.selectedIndex = data->map.nodeIndex;
					data->map.selectedNode = data->map.node[data->map.selectedIndex];
					data->map.activeNode = data->map.selectedNode;
			break;

		case kGraphPresetTwo :

			// draw 6x6 grid of pins
			for(i = -3; i < 3; i++)
				for(j=-3; j < 3; j++)
				{
					// create the pin trunk
					node = npNodeNew (kNodePin, dataRef);
					node->translate.x = i * 10.0f;
					node->translate.y = j * 10.0f;

					node->translate.x += translateOffset.x;
					node->translate.y += translateOffset.y;
					node->translate.z += translateOffset.z;

					// create child node toroids...
				//	node = node->child[node->childIndex];
/*					npNodeNewChild (node, dataRef);
					npNodeNewChild (node, dataRef);
					npNodeNewChild (node, dataRef);
					node = npNodeNewChild (node, dataRef);
					npNodeNewChild (node, dataRef);
					node = npNodeNewChild (node, dataRef);
					npNodeNewChild (node, dataRef);
					*/
		//			node = npNodeNew (kNodePin, dataRef);
						node->colorIndex = 15;
					nodeChild = node->child[0];
						nodeChild->colorIndex = 14;

					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 14;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 2;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 3;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 4;
					data->map.nodeIndex = data->map.nodeCount - 1;
					data->map.selectedIndex = data->map.nodeIndex;
					data->map.selectedNode = data->map.node[data->map.selectedIndex];
					data->map.activeNode = data->map.selectedNode;


				}			
			break;
		case kGraphPresetThree :
node = npNodeNew (kNodePin, dataRef);
											node->rotateRate.z = 0.001f;
						node->colorIndex = 15;
					nodeChild = node->child[0];
						nodeChild->colorIndex = 14;

					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 14;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 2;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 3;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 4;

node = npNodeNew (kNodePin, dataRef);
											node->rotateRate.z = -0.001f;
						node->colorIndex = 15;
					nodeChild = node->child[0];
						nodeChild->colorIndex = 14;

					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 14;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 2;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 3;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 4;

node = npNodeNew (kNodePin, dataRef);
						node->colorIndex = 15;
					nodeChild = node->child[0];
						nodeChild->colorIndex = 15;

					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 14;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
										node->rotateRate.x = 0.001f;
						node->colorIndex = 2;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
											node->rotateRate.x = 0.005f;
						node->colorIndex = 3;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 4;

node = npNodeNew (kNodePin, dataRef);
											node->rotateRate.z = 0.005f;
						node->colorIndex = 15;
					nodeChild = node->child[0];
						nodeChild->colorIndex = 14;

					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 14;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 2;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 2;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 3;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 4;

node = npNodeNew (kNodePin, dataRef);
											node->rotateRate.z = 0.001f;
						node->colorIndex = 15;
					nodeChild = node->child[0];
						nodeChild->colorIndex = 14;

					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 14;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 2;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 3;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 4;

					npCtrlCommand (kGraphSelectAll, data);
					npCtrlCommand (kGraphColorUp, data);
					npCtrlCommand (kGraphColorDown, data);
					npCtrlCommand (kGraphSelectAll, data);

						
					data->map.nodeIndex = data->map.nodeCount - 1;
					data->map.selectedIndex = data->map.nodeIndex;
					data->map.selectedNode = data->map.node[data->map.selectedIndex];
					data->map.activeNode = data->map.selectedNode;

					npCtrlCommand (kGraphNext, data);
					npCtrlCommand (kGraphNext, data);
					npCtrlCommand (kGraphNext, data);
					npCtrlCommand (kGraphNext, data);
	
			break;
		case kGraphPresetFour : 
					node = npNodeNew (kNodePin, dataRef);
										node->rotateRad.z = 1.7f;
										node->scale.x = 1.5f;
										node->scale.y = 1.5f;
										node->scale.z = 1.5f;

										node->translate.x -= 3.0f;
										node->translate.y -= 22.0f;

						node->colorIndex = 15;
					nodeChild = node->child[0];
						nodeChild->colorIndex = 14;
/*
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 1;
	*/			nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 15;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 15;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 1;
					nodeChild = npNodeNewChild (node, dataRef);
						nodeChild->colorIndex = 14;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 4;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 3;
										node->rotateRad.x = -0.7f;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 3;
										node->rotateRad.x = 0.7f;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 4;
										node->rotateRad.x = 0.7f;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 2;
										node->rotateRad.x = 0.7f;
					node = npNodeNewChild (node, dataRef);
						node->colorIndex = 1;
										node->rotateRad.x = 1.5f;


					npCtrlCommand (kGraphSelectAll, data);
					npCtrlCommand (kGraphColorUp, data);
					npCtrlCommand (kGraphColorDown, data);
					npCtrlCommand (kGraphSelectAll, data);

					data->map.nodeIndex = data->map.nodeCount - 1;
					data->map.selectedIndex = data->map.nodeIndex;
					data->map.selectedNode = data->map.node[data->map.selectedIndex];
					data->map.activeNode = data->map.selectedNode;
			break;

		case kGraphPresetFive : break;
		case kGraphPresetSix : break;
		case kGraphPresetSeven : break;
		case kGraphPresetEight : break;

		default : break;
	}

//	data->map.nodeIndex = 1;			// select the camera
}


//------------------------------------------------------------------------------
void npDataCameraReset (NPnodePtr node, void* dataRef)
{
	npInitNodeDefault (node);
	npInitNodeCamera (node);

	node->translate.x = -27.0f;
	node->translate.y = -25.0f;
	node->translate.z = 9.0f;
	
	node->rotate.angle = 0.0f;
	node->rotate.x = 0.0f;
	node->rotate.y = 1.0f;
	node->rotate.z = 0.0f;

	node->rotateRad.x = kPos2PI / 32.0f;
	node->rotateRad.z = kPos2PI / 12.0f;

//	node->triggerOnLow.z = true;				//prevents going below surface
//	node->triggerLevelLow.z = 1.5f;
}

//------------------------------------------------------------------------------
void npDataCameraResetTwo (NPnodePtr node, void* dataRef)
{
	npInitNodeDefault (node);
	npInitNodeCamera (node);

	node->translate.x = -0.0f;
	node->translate.y = -0.0f;
	node->translate.z = 100.0f;
	
	node->rotate.angle = 0.0f;
	node->rotate.x = 0.0f;
	node->rotate.y = 1.0f;
	node->rotate.z = 0.0f;

	node->rotateRad.x = kPos2PI / 4.0f;

//	node->triggerOnLow.z = true;				//prevents going below surface
//	node->triggerLevelLow.z = 1.5f;
}

//------------------------------------------------------------------------------
void npDataCameraResetThree (NPnodePtr node, void* dataRef)
{
	npInitNodeDefault (node);
	npInitNodeCamera (node);

	node->translate.x = -0.0f;
	node->translate.y = -85.0f;
	node->translate.z = 7.0f;
	
	node->rotate.angle = 0.0f;
	node->rotate.x = 0.0f;
	node->rotate.y = 1.0f;
	node->rotate.z = 0.0f;

//	node->triggerOnLow.z = true;				//prevents going below surface
//	node->triggerLevelLow.z = 1.5f;
}

//------------------------------------------------------------------------------
void npDataCameraResetFour (NPnodePtr node, void* dataRef)
{
	npInitNodeDefault (node);
	npInitNodeCamera (node);

	node->translate.x = 85.0f;
	node->translate.y = 0.0f;
	node->translate.z = 7.0f;
	
	node->rotate.angle = 0.0f;
	node->rotate.x = 0.0f;
	node->rotate.y = 1.0f;
	node->rotate.z = 0.0f;

	node->rotateRad.z = -kPos2PI / 4.0f;

//	node->triggerOnLow.z = true;				//prevents going below surface
//	node->triggerLevelLow.z = 1.5f;
}
